@@ -6,7 +6,7 @@ __classification__ = "UNCLASSIFIED"
 __author__ = "Thomas McCullough"
 
 
-from typing import List
+from typing import List, Union, Dict, Optional
 
 from sarpy.io.xml.base import Serializable, ParametersCollection
 from sarpy.io.xml.descriptors import StringDescriptor, StringEnumDescriptor, \
@@ -31,13 +31,17 @@ class RadarModeType(Serializable):
         'ModeID', _required, strict=DEFAULT_STRICT,
         docstring='Radar imaging mode per Program Specific Implementation Document.')  # type: str
 
-    def __init__(self, ModeID=None, ModeType=None, **kwargs):
+    def __init__(
+            self,
+            ModeType: str = None,
+            ModeID: Optional[str] = None,
+            **kwargs):
         """
 
         Parameters
         ----------
-        ModeID : str
         ModeType : str
+        ModeID : None|str
         kwargs
         """
 
@@ -45,10 +49,11 @@ class RadarModeType(Serializable):
             self._xml_ns = kwargs['_xml_ns']
         if '_xml_ns_key' in kwargs:
             self._xml_ns_key = kwargs['_xml_ns_key']
-        self.ModeID, self.ModeType = ModeID, ModeType
+        self.ModeID = ModeID
+        self.ModeType = ModeType
         super(RadarModeType, self).__init__(**kwargs)
 
-    def get_mode_abbreviation(self):
+    def get_mode_abbreviation(self) -> str:
         """
         Get the mode abbreviation for the suggested name.
 
@@ -115,8 +120,17 @@ class CollectionInfoType(Serializable):
         'Parameters', _collections_tags, _required, strict=DEFAULT_STRICT,
         docstring='Free form parameters object collection.')  # type: ParametersCollection
 
-    def __init__(self, CollectorName=None, IlluminatorName=None, CoreName=None, CollectType=None,
-                 RadarMode=None, Classification="UNCLASSIFIED", CountryCodes=None, Parameters=None, **kwargs):
+    def __init__(
+            self,
+            CollectorName: str = None,
+            IlluminatorName: Optional[str] = None,
+            CoreName: str = None,
+            CollectType: Optional[str] = None,
+            RadarMode: RadarModeType = None,
+            Classification: str = "UNCLASSIFIED",
+            CountryCodes: Union[str, List[str]] = None,
+            Parameters: Union[ParametersCollection, Dict] = None,
+            **kwargs):
         """
 
         Parameters
@@ -136,9 +150,12 @@ class CollectionInfoType(Serializable):
             self._xml_ns = kwargs['_xml_ns']
         if '_xml_ns_key' in kwargs:
             self._xml_ns_key = kwargs['_xml_ns_key']
-        self.CollectorName, self.IlluminatorName = CollectorName, IlluminatorName
-        self.CoreName, self.CollectType = CoreName, CollectType
+        self.CollectorName = CollectorName
+        self.IlluminatorName = IlluminatorName
+        self.CoreName = CoreName
+        self.CollectType = CollectType
         self.RadarMode = RadarMode
         self.Classification = Classification
-        self.CountryCodes, self.Parameters = CountryCodes, Parameters
+        self.CountryCodes = CountryCodes
+        self.Parameters = Parameters
         super(CollectionInfoType, self).__init__(**kwargs)
