@@ -72,7 +72,7 @@ class ExploitationCalculator(object):
         vec_norm = numpy.linalg.norm(vec)
         if vec_norm < 1e-6:
             logger.error(
-                'input vector to be normlaized has norm {}, this may be a mistake'.format(vec_norm))
+                'input vector to be normalized has norm {}, this may be a mistake'.format(vec_norm))
         return vec/vec_norm
 
     # The geometry properties
